import tinycolor from 'tinycolor2'

function _colorChange(data, oldHue) {
    if (data.a && data.a > 1) {
        data.a = 1
    }

    data.a = data.a || 1

    let color
    if (data.hex) {
        color = tinycolor(data.hex)
        color.setAlpha(data.a)
    } else {
        color = tinycolor(data)
    }
    const hsl = color.toHsl()
    const hsv = color.toHsv()

    // 如果引用组件的地方调用了change-color,也触发了这个，实际上data是个包含hsl等属性的大对象
    if (hsl.s === 0) {
        if (data.hsl) {
            hsl.h = data.h || data.hsl.h || oldHue || 0
            hsv.h = data.h || data.hsv.h || oldHue || 0
        } else {
            hsl.h = data.h || oldHue || 0
            hsv.h = data.h || oldHue || 0
        }
    }

    // 对hsl进行处理，h保留整数，s,l保留二位小数，
    hsl.h = Math.round(hsl.h)
    hsl.s = hsl.s.toFixed(2)
    hsl.l = hsl.l.toFixed(2)

    return {
        hsl,
        hsv,
        hex: color.toHexString().toUpperCase(),
        rgba: color.toRgb(),
        oldHue: data.h || oldHue || hsl.h,
        source: data.source,
        a: data.a
    }
}

export default {
    props: ['value'],
    data() {
        return {
            val: _colorChange(this.value)
        }
    },
    computed: {
        colors: {
            get() {
                return this.val
            },
            set(newVal) {
                this.val = newVal
                this.$emit('change-color', newVal)
                this.$emit('input', newVal)
            }
        }
    },
    watch: {
        value(newVal) {
            this.val = _colorChange(newVal)
            // this.$emit('change', newVal)
        }
    },
    created() {
        // console.log(this.colors)
        /*
          Enforce the colorChange in case only HEX value is given.
          Guarantees that HEX value is uppercase and other values such
          as HSL or HSV exists and reflect the HEX value
        */
        // this.colors = _colorChange(this.colors)
    },
    methods: {
        colorChange(data, oldHue) {
            this.colors = _colorChange(data, oldHue || this.oldHue)
            this.oldHue = this.colors.hsl.h
        },
        isValidHex(hex) {
            return tinycolor(hex).isValid()
        },
        simpleCheckForValidColor(data) {
            const keysToCheck = ['r', 'g', 'b', 'a', 'h', 's', 'a', 'v']
            let checked = 0
            let passed = 0

            for (let i = 0; i < keysToCheck.length; i++) {
                const letter = keysToCheck[i]
                if (data[letter]) {
                    checked++
                    if (!isNaN(data[letter])) {
                        passed++
                    }
                }
            }
            // debugger
            if (checked === passed) {
                return data
            }
            return null
        }
    }
}
