import AdGrid from '../components/AdGrid'
import Banner from '../components/Banner'
import Layout from '../components/Layout'
import MakeCall from '../components/MakeCall'
import MenuGrid from '../components/MenuGrid'
import Page from '../components/Page'
import TabMenu from '../components/TabMenu'
import LinkTo from '../components/LinkTo'
import Placeholder from '../components/Placeholder'
import Separator from '../components/Separator'
import Tip from '../components/Tip'
import HeaderLink from '../components/HeaderLink'
import HeaderMenu from '../components/HeaderMenu'

import Emulator from './emulator'
import EmulatorCompView from './preview/compView'
import EmulatorCompLayout from './preview/compLayout'
import EmulatorCompSortable from './preview/compSortable'

import EmulatorStoreModule from './store'

import axios from './utils/axios'
import CommonSDK from './plugins/CommonSDK'

export let _Vue; // eslint-disable-line

export default {
    install(Vue, options) {
        _Vue = Vue;

        const inject = options.inject || {};
        const router = options.router;

        window.$app = {
            axios: axios.init(router),
            appId: inject.appid || '',
            apiDomain: inject.apiDomain || '',
            fetchMetaApi: inject.fetchMetaApi || '',
            fetchVersionApi: inject.fetchVersionApi || '',
            appAssetsVersion: inject.appVersion || '',
            appVersion: '',
            exception() {
            },
            message() {
            }
        };

        Vue.use(CommonSDK, { router });
        Vue.component('page', Page)
        Vue.component('tabmenu', TabMenu)
        Vue.component('layout', Layout)
        Vue.component('banner', Banner)
        Vue.component('menugrid', MenuGrid)
        Vue.component('adgrid', AdGrid)
        Vue.component('makecall', MakeCall)
        Vue.component('linkto', LinkTo)
        Vue.component('placeholder', Placeholder)
        Vue.component('separator', Separator)
        Vue.component('headerlink', HeaderLink)
        Vue.component('headermenu', HeaderMenu)

        Vue.component('tip', Tip)

        Vue.component('emulator', Emulator)
        Vue.component('comp-view', EmulatorCompView)
        Vue.component('comp-layout', EmulatorCompLayout)
        Vue.component('comp-sortable', EmulatorCompSortable)

        const rootStore = options.rootStore
        if (rootStore) {
            rootStore.registerModule('emulator', EmulatorStoreModule)
        }
    }
}
