const win = window;
const EXTERNAL = 'external';
const INTERNAL = 'internal';

const utils = {
    push(arg) {
        arg && win.location.assign(arg.url)
    },
    replace(arg) {
        arg && win.location.replace(arg.url)
    }
};

const normal = {
    push() {
    },
    replace() {
    }
};

function wrapper(location) {
    const type = typeof location;
    if (type === 'string') {
        if (/^https?:\/\//i.test(location)) {
            return {
                type: EXTERNAL,
                url: location
            }
        }
        return {
            type: INTERNAL,
            name: location
        }
    } else if (type === 'object' && !location.type) {
        if (location.name) {
            location.type = INTERNAL
        }
        if (location.url) {
            location.type = EXTERNAL
        }
    }
    return location || {}
}

function proxy(arg, access) {
    if (document.querySelector('.emulator-preview')) {
        return normal
    }
    if (arg.type === EXTERNAL) {
        return utils
    }
    if (arg.type === INTERNAL) {
        return access || normal
    }
    return normal
}

export default {
    init(Vue, router) {
        Object.defineProperty(Vue.prototype, '$location', {
            get: () => ({
                push(arg) {
                    const _arg = wrapper(arg);
                    proxy(_arg, router).push(_arg);
                },
                replace(arg) {
                    const _arg = wrapper(arg);
                    proxy(_arg, router).replace(_arg);
                }
            })
        })
    }
}
