import * as T from './types';
import {
    getViewItems,
    mapRoute,
    updateDesignMeta,
    updateDesignMetaByAddComp,
    updateDesignMetaByAddTab,
    updateDesignMetaByDeleteTab,
    updateDesignMetaByEditTab,
    updateDesignMetaBySortItems,
    updateDesignMetaBySortTabs
} from '../utils/metaHandle'

export default {

    // 更新App元数据
    [T.UPDATE_DESIGN_META](state, payload) {
        if (payload) {
            state.design_meta = payload;
            state.currViewItems = getViewItems(state.design_meta, state.currViewName)
        }
    },

    // 更新当前视图名称
    [T.UPDATE_CURRVIEW_NAME](state, payload) {
        if (payload) {
            state.currViewName = payload;
            state.currViewItems = getViewItems(state.design_meta, state.currViewName)
        }
    },

    // 设置当前视图下的内容Items
    [T.UPDATE_VIEW_ITEMS](state, payload) {
        if (payload) {
            state.currViewItems = payload;
            state.design_meta = updateDesignMetaBySortItems(state.design_meta, state.currViewName, payload)
        }
    },

    // 注册组件
    [T.UPDATE_COMPS](state, payload) {
        if (payload) {
            const components = payload.concat();
            const comps = {};
            components.forEach((component) => {
                comps[component.id] = {
                    id: component.id,
                    previewData: (component.compData && component.compData.designer && component.compData.designer.previewData) || {}
                }
            });
            state.design_components = comps
        }
    },

    // 当前视图添加组件
    [T.UPDATE_VIEW_COMP](state, payload) {
        if (payload) {
            state.design_meta = updateDesignMetaByAddComp(state.design_meta, state.currViewName, state.design_components, payload.compId, payload.newIndex, payload.layoutKey, payload.layoutItemIndex);
            state.currViewItems = getViewItems(state.design_meta, state.currViewName);
            state.design_selectComp_meta = {}
        }
    },

    // 更新当前选择的组件
    [T.UPDATE_SELECT_COMP](state, payload) {
        if (payload) {
            state.design_selectComp_meta = payload
        }
    },

    // 删除组件
    [T.DELETE_COMP](state, payload) {
        if (payload) {
            state.design_meta = updateDesignMeta(state.design_meta, state.currViewName, payload._key);
            state.currViewItems = getViewItems(state.design_meta, state.currViewName);
            state.design_selectComp_meta = {};
        }
    },

    // 排序导航
    [T.UPDATE_TABS_SORT](state, payload) {
        if (payload) {
            state.design_meta = updateDesignMetaBySortTabs(state.design_meta, payload)
        }
    },

    // 删除导航
    [T.DELETE_TAB](state, payload) {
        if (payload) {
            state.design_meta = updateDesignMetaByDeleteTab(state.design_meta, payload);
            if (state.currViewName === payload.href.name && state.design_meta.tabs.items.length > 0) {
                state.currViewName = state.design_meta.tabs.items[0].href.name;
                state.currViewItems = getViewItems(state.design_meta, state.currViewName)
            }
            if (!state.design_meta.tabs.items.length) {
                state.currViewName = '';
                state.currViewItems = []
            }
        }
    },

    // 新增/修改导航
    [T.DO_ADDOREDIT_TAB](state, payload) {
        const { tab, index, viewMeta, linkPages } = payload;
        if (payload) {
            if (payload.tab.isAdd) {
                state.design_meta = updateDesignMetaByAddTab(state.design_meta, tab, index, viewMeta, linkPages)
            } else {
                state.design_meta = updateDesignMetaByEditTab(state.design_meta, tab, index, viewMeta, linkPages)
            }
        }
    },

    [T.UPDATE_PAGE_LIST](state, payload) {
        if (payload) {
            const routeMap = {};

            mapRoute(payload, routeMap);

            state.pages = Array.isArray(payload) ? payload : [payload];
            state.routeMap = routeMap;
        }
    }
}
