import axios from 'axios'

export default {
    init(router) {
        const instance = axios.create();

        instance.interceptors.request.use((config) => {
            const data = instance.getMockData(config.url.replace(config.baseURL || '', ''));
            if (data) {
                config.adapter = function () {
                    return Promise.resolve({ data, config })
                }
            }
            return config;
        }, error => Promise.reject(error));

        instance.interceptors.response.use(response => response, (error) => {
            if (router && error.response && error.response.status === 401) {
                try {
                    window.localStorage.removeItem('appRole');
                    router.replace({
                        name: 'login',
                        path: '/login',
                        query: {
                            logout: true
                        }
                    });
                } catch (e) {
                    //
                } finally {
                    window.location.reload()
                }
            }
            return Promise.reject(error);
        });

        instance.all = axios.all;
        instance.spread = axios.spread;

        instance.addMockData = (items) => {
            if (items) {
                instance._mock = instance._mock || {};
                if (Array.isArray(items)) {
                    items.forEach((item) => {
                        if (item.url) {
                            instance._mock[item.url] = item.data || {}
                        }
                    })
                } else if (typeof items === 'object') {
                    Object.keys(items).forEach((url) => {
                        if (url) {
                            instance._mock[url] = items[url] || {}
                        }
                    })
                }
            }
        };

        instance.getMockData = url => instance._mock && instance._mock[url];

        return instance
    }
}
