import cloneDeep from 'lodash/cloneDeep'
import { _Vue } from '../index'
import editComponent from '../edit/index'


// 准备组件数据
export function prepareComponents(components, editComps, bizComps) {
    return components.map((component) => {
        if (component.type === 'biz') {
            const _biz = bizComps.find(biz => biz.id === component.id);
            return {
                ...component,
                compData: _biz ? cloneDeep(_biz) : {}
            }
        } else if (component.type === 'base') {
            return {
                ...component,
                compData: editComps[component.id]
            }
        }
        return {}
    });
}

// 注册组件
export function registerComponents(components) {
    components.forEach((comp) => {
        if (comp.compData) {
            if (comp.compData.component) {
                _Vue.component(comp.id, comp.compData.component)
            }

            if (comp.compData.designer && comp.compData.designer.ctrl) {
                _Vue.component(`${comp.id}-ctrl`, {
                    props: {
                        compMeta: Object,
                        linkPages: Object
                    },
                    render(h) {
                        const self = this;
                        return h(editComponent, {
                            props: {
                                id: comp.id,
                                designer: comp.compData.designer,
                                compMeta: self.compMeta,
                                linkPages: self.linkPages
                            },
                            on: {
                                onupdate(compMeta) {
                                    self.$emit('onupdate', compMeta)
                                },
                                onUpdateLink(e) {
                                    self.$emit('onUpdateLink', e)
                                }
                            }
                        })
                    }
                })
            }
        }
    })
}

export function registerSystemComponents(components) {
    components.forEach((comp) => {
        if (comp) {
            if (comp.component) {
                _Vue.component(comp.id, comp.component)
            }

            if (comp.designer) {
                _Vue.component(`${comp.id}-ctrl`, {
                    props: {
                        compMeta: Object
                    },
                    //保留和基础、业务组件注册相同结构
                    render(h) {
                        const self = this;
                        return h(editComponent, {
                            props: {
                                id: comp.id,
                                designer: comp.designer,
                                compMeta: self.compMeta
                            },
                            on: {
                                onupdate(compMeta) {
                                    self.$emit('onupdate', compMeta)
                                },
                                onUpdateLink(e) {
                                    self.$emit('onUpdateLink', e)
                                }
                            }
                        })
                    }
                })
            }
        }
    })
}
