import isEqual from 'lodash/isEqual';
import cloneDeep from 'lodash/cloneDeep';

// 为组件添加key
window._compCounter = window._compCounter || 1;

// 视图根属性列表
const viewProps = ['name', 'path', 'body', 'style', 'isLogin'];

function getCompCounter() {
    return window._compCounter++;
}

export function addCompKey(component) {
    const _key = `comp-${getCompCounter()}`;
    if (!component._key) {
        component._key = _key;
    }
}

export function addLayoutKey(layout) {
    const _key = `layout-${getCompCounter()}`;
    if (!layout._key) {
        layout._key = _key;
    }
}

// 更新元数据- 通过编辑/删除组件
function updateItemMeta(compItem, compKey, compMeta) {
    let item = { ...compItem };
    if (item._key === compKey) {
        item = {
            ...item,
            ...compMeta,
            _key: item._key
        }
    } else if (item.type === 'horizontal' || item.type === 'vertical') {
        item.components = item.components && item.components.map(component => updateItemMeta(component, compKey, compMeta))
    }
    return item
}

function filterItems(items, compKey) {
    const _items = [];
    items.forEach((item) => {
        if (item._key !== compKey) {
            if (item.type === 'horizontal' || item.type === 'vertical') {
                item.components = filterItems(item.components, compKey);
            }
            _items.push(item)
        }
    });
    return _items;
}

export function updateViewMeta(designMeta, viewName, viewMeta) {
    const newDesignMeta = cloneDeep(designMeta);
    const currentViewMeta = newDesignMeta.views.find(view => view.name === viewName);
    if (currentViewMeta && currentViewMeta.body) {
        currentViewMeta.style = viewMeta.view.style;
        currentViewMeta.body = viewMeta.view.body
    }
    return newDesignMeta
}

export function updateDesignMeta(designMeta, viewName, compKey, compMeta) {
    const _designMeta = { ...designMeta };
    const viewMeta = _designMeta.views.find(view => view.name === viewName);

    if (viewMeta && viewMeta.body) {
        const _content = viewMeta.body.content
        if (_content && _content.items && Array.isArray(_content.items)) {
            if (compMeta) { //编辑组件
                _content.items.forEach((item, i) => {
                    _content.items[i] = updateItemMeta(item, compKey, compMeta)
                })
            } else { //删除组件
                _content.items = filterItems(_content.items, compKey)
            }
        }
        // header 左右菜单组件
        const _header = viewMeta.body.header
        if (_header && _header.items && Array.isArray(_header.items)) {
            if (compMeta) { //编辑组件
                _header.items.forEach((item, i) => {
                    _header.items[i] = updateItemMeta(item, compKey, compMeta)
                })
            } else { //删除组件
                _header.items = cloneDeep(_header.items).filter(item => item._key !== compKey)
            }
        }
    }
    return _designMeta;
}

// 更新元数据-通过排序视图内容
export function updateDesignMetaBySortItems(designMeta, viewName, bodyItems) {
    const _designmeta = { ...designMeta };
    const viewMeta = _designmeta.views && _designmeta.views.find(view => view.name === viewName);
    viewMeta.body.content.items = bodyItems.concat();
    return _designmeta;
}

// 更新元数据-通过视图添加组件
function addLayoutComp(items, layoutKey, layoutItemIndex, comp) {
    const _items = [];
    items.forEach((item) => {
        if (item.type === 'horizontal' || item.type === 'vertical') {
            if (item._key === layoutKey) {
                let len = item.components.length
                if (layoutItemIndex + 1 > len) {
                    len = layoutItemIndex + 1
                }
                len = len > 2 ? len : 2
                let _components = Array(...Array(len))
                _components = _components.map((_component, index) => {
                    if (index === layoutItemIndex) {
                        return comp
                    }
                    const layoutItemIndexComponent = item.components.find(component => component._layoutItemIndex === index)
                    return layoutItemIndexComponent || item.components[index] || {}
                })
                item.components = _components
            } else {
                item.components = addLayoutComp(item.components, layoutKey, layoutItemIndex, comp);
            }
        }

        _items.push(item)
    });
    return _items;
}

/*eslint-disable */
function getUuid(len, radix) {
    const chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz'.split('');
    const uuid = [];
    let i;
    radix = radix || chars.length;

    if (len) {
        // Compact form
        for (i = 0; i < len; i++) uuid[i] = chars[0 | Math.random() * radix];
    } else {
        // rfc4122, version 4 form
        let r;

        // rfc4122 requires these characters
        uuid[8] = '-';
        uuid[13] = '-';
        uuid[18] = '-';
        uuid[23] = '-';
        uuid[14] = '4';

        // Fill in random data. At i==19 set the high bits of clock sequence as
        // per rfc4122, sec. 4.1.5
        for (i = 0; i < 36; i++) {
            if (!uuid[i]) {
                r = 0 | Math.random() * 16;
                uuid[i] = chars[(i === 19) ? (r & 0x3) | 0x8 : r];
            }
        }
    }

    return uuid.join('');
}

export function updateDesignMetaByAddComp(designMeta, viewName, designComponents, compId, newIndex, layoutKey, layoutItemIndex) {
    const _designMeta = { ...designMeta };
    const viewMeta = _designMeta.views && _designMeta.views.find(view => view.name === viewName);

    const components = { ...designComponents };
    let previewData = (components[compId] && components[compId].previewData) || {};

    // 替换uuid初始值
    let previewDataString = JSON.stringify(previewData)
    if (previewDataString && previewDataString.indexOf('{{uuid}}') > -1) {
        previewData = JSON.parse(previewDataString.replace(/{{uuid}}/g, getUuid(8, 16)))
    }

    const comp = {
        type: compId,
        ...previewData
    };

    if (previewData.components) {
        comp.components = previewData.components.concat()
    }

    if (!layoutKey) {
        if (newIndex >= 0) {
            viewMeta.body.content.items.splice(newIndex, 0, comp)
        } else {
            viewMeta.body.content.items.push(comp)
        }
    } else {
        viewMeta.body.content.items = addLayoutComp(viewMeta.body.content.items, layoutKey, layoutItemIndex, comp)
    }
    return _designMeta;
}

// 更新元数据 - 修改页面跳转
function findPage(pages, pageRouteId) {
    let findedPage = null;
    for (let i = 0; i < pages.length; i++) {
        const page = pages[i];
        if (page.type === 'folder') {
            if (page.children && page.children.length > 0) {
                findedPage = findPage(page.children, pageRouteId);
                if (findedPage) {
                    break
                }
            }
        } else if (page.id === pageRouteId) {
            findedPage = page;
            break
        }
    }
    return findedPage
}

function findViewHrefs(view) {
    let hrefs = [];
    const viewType = Object.prototype.toString.call(view);

    if (viewType === '[object Object]') {
        const properties = Object.keys(view);
        if (properties && properties.length > 0) {
            for (let i = 0; i < properties.length; i++) {
                if (properties[i] === 'href') {
                    const propVal = view[properties[i]];
                    if (propVal.name) {
                        hrefs.push(propVal.name)
                    }
                } else {
                    hrefs = hrefs.concat(findViewHrefs(view[properties[i]]))
                }
            }
        }
    } else if (viewType === '[object Array]') {
        for (let j = 0; j < view.length; j++) {
            hrefs = hrefs.concat(findViewHrefs(view[j]))
        }
    }
    return hrefs
}

function addDesignMetaView(designMeta, page, linkPages) {
    let views = [];
    if (designMeta) {
        const designmeta = { ...designMeta };
        views = designmeta.views.concat()
    }

    if (!views.find(view => view.name === page.id)) {
        views.push(page.metadata ? JSON.parse(page.metadata) : {
            name: page.id,
            path: `/${page.id}`,
            body: {
                header: {
                    title: page.name
                },
                content: {
                    items: []
                }
            }
        });

        //递归查找添加的页面是否有路由跳转
        if (page.metadata) {
            const addedPageHrefs = findViewHrefs(JSON.parse(page.metadata));
            if (addedPageHrefs && addedPageHrefs.length > 0) {
                for (let i = 0; i < addedPageHrefs.length; i++) {
                    const linkpage = findPage(linkPages, addedPageHrefs[i]);
                    if (linkpage) {
                        views = views.concat(addDesignMetaView(null, linkpage, linkPages))
                    }
                }
            }
        }
    }
    return views
}

function deleteDesignMetaView(designMeta, viewName, designViews) {
    let views = (designMeta && designMeta.views.concat()) || [];
    if (designViews) {
        views = designViews
    }
    let isUseView = false;

    // 判断Tab导航是否指向该页面
    const tabs = designMeta.tabs.items;
    for (let i = 0; i < tabs.length; i++) {
        if (tabs[i].href && tabs[i].href.name && tabs[i].href.name === viewName) {
            isUseView = true;
            break;
        }
    }

    // 查找页面路由
    let viewAllHrefs = [];
    if (!isUseView) {
        for (let j = 0; j < views.length; j++) {
            if (views[j].name !== viewName) {
                viewAllHrefs = viewAllHrefs.concat(findViewHrefs(views[j]))
            }
        }
    }
    if (viewAllHrefs.indexOf(viewName) > -1) {
        isUseView = true
    }

    if (!isUseView) {
        //递归查找删除的页面是否有路由跳转
        const view = views.find(v => v.name === viewName);
        if (view) {
            const viewHrefs = findViewHrefs(view);

            views = views.filter(_view => _view.name !== viewName);

            if (viewHrefs && viewHrefs.length > 0) {
                for (let k = 0; k < viewHrefs.length; k++) {
                    views = deleteDesignMetaView(designMeta, viewHrefs[k], views)
                }
            }
        }
    }
    return views
}

export function updateDesignMetaByChangeViews(designMeta, oldCompHref, compHref, linkPages) {
    const designmeta = { ...designMeta };
    if (!isEqual(oldCompHref, compHref)) {
        if ((!oldCompHref || (oldCompHref && !oldCompHref.name)) && compHref && compHref.name) {
            // 新增路由
            const linkpage = findPage(linkPages, compHref.name);
            if (linkpage) {
                designmeta.views = addDesignMetaView(designmeta, linkpage, linkPages)
            }
        } else if (oldCompHref && oldCompHref.name && (!compHref || (compHref && !compHref.name))) {
            // 删除路由
            designmeta.views = deleteDesignMetaView(designmeta, oldCompHref.name)
        } else if (oldCompHref && oldCompHref.name && compHref && compHref.name && oldCompHref.name !== compHref.name) {
            // 修改路由
            designmeta.views = deleteDesignMetaView(designmeta, oldCompHref.name);
            const linkpage = findPage(linkPages, compHref.name);
            if (linkpage) {
                designmeta.views = addDesignMetaView(designmeta, linkpage, linkPages)
            }
        }
    }
    return designmeta
}

// 更新元数据-通过排序导航菜单
export function updateDesignMetaBySortTabs(designMeta, tabs) {
    const designmeta = { ...designMeta };
    designmeta.tabs.items = tabs.concat();
    return designmeta;
}

// 更新元数据-通过删除导航
export function updateDesignMetaByDeleteTab(designMeta, tab) {
    const designmeta = { ...designMeta };
    designmeta.tabs.items = designmeta.tabs.items.filter(t => t.text !== tab.text);
    designmeta.views = deleteDesignMetaView(designmeta, tab.href.name);
    return designmeta;
}

// 更新元数据-通过新增导航
export function updateDesignMetaByAddTab(designMeta, tab, index, viewMeta, linkPages) {
    const designmeta = { ...designMeta };
    tab.href = {
        name: viewMeta.name
    };
    designmeta.tabs.items.push(tab);

    const linkpage = findPage(linkPages, viewMeta.name);
    if (linkpage) {
        designmeta.views = addDesignMetaView(designmeta, linkpage, linkPages)
    }
    return designmeta;
}

// 更新元数据-通过编辑导航
export function updateDesignMetaByEditTab(designMeta, tab, index, viewMeta, linkPages) {
    const designmeta = { ...designMeta };
    designmeta.tabs.items = designmeta.tabs.items.map((item, i) => {
        if (i === +index) {
            return {
                ...tab,
                href: {
                    name: viewMeta.name
                }
            }
        }
        return item
    });
    designmeta.views = deleteDesignMetaView(designmeta, tab.href.name);

    const linkpage = findPage(linkPages, viewMeta.name);
    if (linkpage) {
        designmeta.views = addDesignMetaView(designmeta, linkpage, linkPages)
    }
    return designmeta;
}

// 获取视图内容
export function getViewItems(designMeta, viewName) {
    const designmeta = { ...designMeta };
    const viewMeta = designmeta.views && designmeta.views.find(view => view.name === viewName);
    return viewMeta && viewMeta.body && viewMeta.body.content && viewMeta.body.content.items ? viewMeta.body.content.items : []
}

/**
 * 自定义页面数据、视图ids，找到视图列表
 * @param data
 * @param ids 传入空返回全部view
 * @param views
 * @returns {*|Array}
 */
export function findViewsByIds(data, ids, views) {
    views = views || [];
    if (data && (!ids || ids.length)) {
        if (!Array.isArray(data) && typeof data === 'object') {
            if (data.type === 'page') {
                const index = ids ? ids.indexOf(data.id) : 0;
                // page没有子层级
                if (index > -1) {
                    if (ids) {
                        ids.splice(index, 1);
                    }
                    let view = {
                        body: {
                            header: {
                                title: data.name
                            },
                            content: {
                                items: []
                            }
                        }
                    };
                    if (data.metadata) {
                        try {
                            view = JSON.parse(data.metadata)
                        } catch (e) {
                            //
                        }
                    }

                    view.name = data.id;
                    view.path = `/${data.id}`;

                    views.push(view);
                }
            } else {
                findViewsByIds(data.children, ids, views)
            }
        } else {
            data.forEach((child) => {
                findViewsByIds(child, ids, views)
            })
        }
    }
    return views
}

/**
 * 找到导航tabs指向的视图Ids，拼接到传入数组
 * @param tabs meta.tabs
 * @param viewIds
 */
export function findTabsViewIds(tabs, viewIds) {
    viewIds = viewIds || [];
    if (tabs && tabs.items) {
        tabs.items.forEach((item) => {
            if (item.href && item.href.name && viewIds.indexOf(item.href.name) === -1) {
                viewIds.push(item.href.name)
            }
        })
    }
    return viewIds
}

/**
 * 找到组件路由指向的视图Ids，拼接到传入数组
 * @param data meta.views
 * @param viewIds
 * @param viewMap
 * @returns {*|Array}
 */
export function findViewIds(data, viewIds, viewMap) {
    viewIds = viewIds || [];
    viewMap = viewMap || {};
    if (data) {
        if (Array.isArray(data)) {
            data.forEach((item) => {
                findViewIds(item, viewIds, viewMap)
            })
        } else if (typeof data === 'object') {
            Object.keys(data).forEach((prop) => {
                if (prop === 'href' && data.href) {
                    let viewId = data.href.name || '';
                    //data.type === 'inner' && data.href.url 临时兼容处理，后续确定方案后移除
                    if (data.type === 'inner' && data.href.url) {
                        viewId = data.href.url
                    }
                    //最新规则 internal\external
                    if (data.href.type === 'internal' && data.href.name) {
                        viewId = data.href.name
                    }

                    if (viewId && viewIds.indexOf(viewId) === -1) {
                        viewIds.push(viewId);
                        //遍历子视图
                        findViewIds(viewMap[viewId], viewIds, viewMap)
                    }
                } else {
                    findViewIds(data[prop], viewIds, viewMap)
                }
            })
        }
    }
    return viewIds
}

/**
 * 保持view清洁
 * @param view
 */
export function cleanView(view) {
    if (view) {
        Object.keys(view).forEach((prop) => {
            if (viewProps.indexOf(prop) === -1) {
                delete view[prop]
            }
        })
    }
    return view
}

/**
 * 保持属性值清洁
 * @param view
 * @param viewIds
 */
export function cleanProp(view, viewIds) {
    if (view) {
        if (Array.isArray(view)) {
            view.forEach((item) => {
                cleanProp(item, viewIds)
            })
        } else if (typeof view === 'object') {
            Object.keys(view).forEach((prop) => {
                if (prop === 'href' && view.href) {
                    if (view.href.type === 'internal' && viewIds.indexOf(view.href.name) === -1) {
                        view.href = {}
                    }
                } else {
                    cleanProp(view[prop], viewIds)
                }
            })
        }
    }
}

/**
 * map viewId=>view方便后续使用
 * @param pages
 * @param currentViews
 * @returns {{}}
 */
export function mapViews(pages, currentViews) {
    const viewMap = {};
    const views = findViewsByIds(pages);
    if (views) {
        views.forEach((view) => {
            if (view) {
                viewMap[view.name] = view
            }
        })
    }
    //已存在于元数据的优先使用
    if (currentViews) {
        currentViews.forEach((view) => {
            if (view && viewMap[view.name]) {
                if (viewMap[view.name].isLogin) {
                    view.isLogin = true
                } else {
                    delete view.isLogin
                }
                viewMap[view.name] = view
            }
        })
    }

    return viewMap
}

/**
 * 过滤出所有依赖视图
 * @param meta
 * @param pages
 * @returns {*}
 */
export function findViews(meta = {}, pages = []) {
    // map viewId=>view
    const viewMap = mapViews(pages, meta.views);
    // find tabsViewId
    const tabViewIds = findTabsViewIds(meta.tabs);
    // find loginViewId
    const loginViewIds = Object.values(viewMap).filter(view => view.isLogin && tabViewIds.indexOf(view.name) === -1).map(view => view.name);
    // concat tabsViewId & loginViewId
    const concatIds = tabViewIds.concat(loginViewIds);
    // find tabsView
    const tabViews = concatIds.map(id => viewMap[id]);
    // find viewIds through tabsView & loginView
    const viewIds = findViewIds(tabViews, concatIds, viewMap);
    // find views
    const views = viewIds.map(id => cleanView(viewMap[id])).filter(view => !!view);
    // clear invalid props
    cleanProp(views, views.map(view => view.name));
    return views
}

export function mapRoute(data, routeMap) {
    if (Array.isArray(data)) {
        for (let i = 0; i < data.length; i++) {
            mapRoute(data[i], routeMap);
        }
    } else if (typeof data === 'object') {
        if (data.type === 'page') {
            routeMap[data.id] = data.name;
        } else if (data.children) {
            mapRoute(data.children, routeMap)
        }
    }
}
